#coding:utf-8
"""Модуль sens содержит классы-обертки для датчиков, подключенных
к шинам 1-Wire и I2C микрокомпьютера Raspberry Pi
История изменений - в конце файла
OldBean, 17.03.01"""
import os, time, threading
import smbus
import Adafruit_BMP.BMP085 as bmp

#-------------------------------------------------------------------------------
class DS18B20(object): # Класс датчика температуры DS18B20
  """     Класс-обертка для датчиков DS18B20, подключенных к шине 1-Wire.
     Температура каждого объекта класса DS18B20 представляется свойством T.
     Цикл преобразования датчика запускается каждый раз, когда это
     свойство используется в правой части выражения.
     Класс DS18B20 также представляет средства для коллективной работы с 
     датчиками: список всех датчиков, подключенных к шине 1-Wire (стаический
     метод IDs() и словарь, содержащий ключи - идентификаторы датчиков и 
     значения - температуры соответствующих датчиков (статический метод Ts())"""
  path = "/sys/bus/w1/devices" # Директория с файлами датчиков
  tsd = {} # Словарь для коллективной работы с датчиками {id датчика : температура}

  def __new__(cls, id):
    """ Перед созданием объекта В методе __new__ производится проверка
        наличия датчика DS18B20 с заданным id"""
    dn = cls.path + ("/28-%012x" % (id))
    if os.path.isdir(dn):
      return super(DS18B20, cls).__new__(cls) 
    else: 
      print "Датчик c id = 0x%012x  отсутствует или неисправен" % (id)
      return None

  def __init__(self, id):
    """id - идентификатор датчика (число)"""
    self._id = id
    self._fn = DS18B20.path + ("/28-%012x/w1_slave" % (id))

  @staticmethod
  def temperature(fn, id = -1):
    """       Запускается цикл преобразования одного датчика DS18B20.
       Если идентификатор датчика не установлен (объект по умолчанию),
       то просто возвращается значение температуры.
       Если же идентификатор датчика (параметр id) задан, то значение
       температуры записывается в словарь tsd. Ключом является id датчика.
       При таком режиме работы предполагается параллельная выполнение циклов
       преобразований датчиков."""
    lines = ["NoNoNo", ""]
    while not lines[0].rstrip().endswith("YES"): # Пока данные не будут готовы
      f = open(fn, "r")
      lines = f.readlines()
      f.close()
    pos = lines[1].find('t=') # Температура - после "t="
    temp = -273.15 # :)
    if pos != -1:
      temp = float(lines[1][pos + 2:])/1000 # Температура в Цельсиях
    if id < 0: return temp
    DS18B20.tsd[id] = temp

  @property
  def T(self):
    """      Свойство - температура датчика. Цикл преобразования 
       (около 750 мс) запускается при каждом использовании этого 
       свойства в правой части выражения."""
    return DS18B20.temperature(self._fn)

  @staticmethod
  def IDs():
    """     Возвращает список идентификаторов всех датчиков DS18B20,
       подключенных к шине 1-Wire."""
    fnl = os.listdir(DS18B20.path)
    idl = []
    for fn in fnl:
      if fn.startswith("28-"): # Директория соответствует датчику DS18B20
        idl.append(int(fn[-12:], 16))
    return idl

  @staticmethod
  def Ts():
    """     Возвращает словарь, содержащий идентификаторы всех датчиков
    DS18B20, подключенных к шине 1-Wire (в качестве ключей словаря), 
    и температуры датчиков (соответствующие значения словаря). Обработка
    каждого датчика производится параллельно в отдельном потоке, поэтому
    суммарное время преобразования всех датчиков существенно меньше, чем
    при последовательном опросе. Например для 4-х датчиков - 0.951 сек
    вместо 3.315 сек."""
    DS18B20.tsd.clear()
    ids = []; fns = []; threads = []
    dnl = os.listdir(DS18B20.path)
    for dn in dnl:
      if dn.startswith("28-"): # Директория соответствует датчику DS18B20
        ids.append(int(dn[-12:], 16)) # Выделяем ID датчика и - в список
        # Формируем полное имя файла с данными от датчика
        fns.append(DS18B20.path + "/" + dn + "/w1_slave")
    nts = len(ids) # Количество датчиков DS18B20, подключенных к шине
    # Создаем классы-потоки для каждого датчика температуры
    for i in range(nts):
      t = threading.Thread(target = DS18B20.temperature, args = (fns[i], ids[i]))
      threads.append(t)
    # Запускаем параллельные потоки циклов преобразований датчиков
    for i in range(nts):
      threads[i].start()
    for i in range(nts): # Ждем завершения всех циклов преобразований датчиков
      threads[i].join()
    return DS18B20.tsd
#-------------------------------------------------------------------------------
class BMP180(object):
  """   Класс-обертка для датчика атмосферного давления и температуры BMP180.
     Давления и температура представлены соответствущими свойствами (P и T)
     экземпляра класса. Чтение датчиков происходит при каждом использовании
     свойств в правой части выражения."""
  sensor = None
  def __new__(cls):
    """ Перед созданием объекта В методе __new__ производится проверка
        наличия и работоспособности датчика BMP180"""
    try:
      cls.sensor = bmp.BMP085()
      cls.sensor.read_pressure()
      cls.sensor.read_temperature()
      return super(BMP180, cls).__new__(cls) 
    except: 
      print "Датчик BMP180 отсутствует или неисправен"
      return None

  def __init__(self):
    self._sensor = BMP180.sensor
  
  @property
  def P(self):
    """Свойство - значение атмосферного давления в мм.рт.ст."""
    return self._sensor.read_pressure()/133.322 # Паскали -> в мм.рт.ст.

  @property
  def T(self):
    """Свойство - значение температуры датчика в градусах Цельсия."""
    return self._sensor.read_temperature()
#-------------------------------------------------------------------------------
class RMS(object):
  """     Класс-обертка для датчика среднеквадратичного напряжения сети.
     По умолчанию предполагается, что датчик подключен к 1-ой шине I2C
     по адресу 5 (можно изменить при создании объекта)."""
  def __new__(cls, addr = 0x05, bus = smbus.SMBus(1)):
    """Перед созданием объекта в методе __new__ производится
       тестирование датчика (наличие и работоспособность).
       addr - адрес датчика на шине I2C. По умолчанию - 5
       bus - шина (объект SMBus). По умолчанию - SMBus(1)"""
    try: # Проверим наличие и работоспособность датчика
      bus.read_byte(addr)
      # Все в порядке - создаем объект - датчик RMS
      return super(RMS, cls).__new__(cls) 
    except: 
      print "Датчик RMS отсутствует или неисправен"
      return None

  def __init__(self, addr = 0x05, bus = smbus.SMBus(1)):
    """addr - адрес датчика на шине I2C. По умолчанию - 5
       bus - шина (объект SMBus). По умолчанию - SMBus(1)"""
    self._addr = addr
    self._bus = bus

  @property
  def V(self):
    """Свойство - значение среднеквадратичного напряжения сети"""
    return self._bus.read_byte(self._addr) + 100
#-------------------------------------------------------------------------------
if __name__ == "__main__": # Тестирование классов модуля sens
  print "\nДатчики DS18B20"
  tsIDs = DS18B20.IDs() # Список ID всех датчиков DS18B20 на шине 1 Wire
  nts = len(tsIDs) # Количество датчиков DS18B20, обнаруженных на шине
  if nts == 0:
    print "Датчики DS18B20 не обнаружены"
  else:
    print "\nПоследовательный опрос %d датчиков" % (nts)
    st = time.time()
    for id in tsIDs:
      ts = DS18B20(id) # Создаем объект (датчик DS18B20)
      temperature = ts.T # Измеряем температуру этим датчиком
      print "0x%012x :% 7.3f°C" % (id, temperature)
    print "%5.3f сек" % (time.time() - st)

    print "\nПараллельный запуск %d датчиков" % (nts)
    st = time.time()
    d = DS18B20.Ts() # Метод Ts() возвращает словарь с id датчиков (ключи) и 
                     # температурой (значения)
    for id in d.keys():
      print "0x%012x :% 7.3f°C" % (id, d[id])
    print "%5.3f сек" % (time.time() - st)

  print "\nДатчик BMP180"
  bmp = BMP180()
  if bmp != None: # Датчик в наличии и функционирует
    print "Давление: \t%5.1f мм.рт.ст." % (bmp.P)
    print "Температура:\t%5.2f°C" % (bmp.T)

  print "\nДатчик RMS"
  vs = RMS()
  if vs != None: # Датчик в наличии и функционируе
    V = vs.V # Измеряем напряжение
    if V > 100: print "Среднеквадратичное напряжение: %d V" % (V)
    else: print "Слишком низкое напряжение в сети ( < 100B)"

  print "\nТест на наличие датчика DS18B20 с id = 0x1888888 :)"
  ts = DS18B20(0x1888888)
  if ts != None: print "Есть такой датчик!!!"

#-------------------------------------------------------------------------------
"""История изменений

25.01.2017 
----------
1. Написана документация модуля
2. Практически полностью переписан класс-обертка DS18B20 и расширен его 
   функционал - добавлена возможность выполнения цикла преобразования
   каждого датчика в параллельных потоках
3. Перед созданием объектов-датчиков (RMS и BMP180) добавлено их тестирование
   (наличие датчика и работоспособность интерфейса).

26.01.2017
----------
4. Перед созданием объект-датчика DS18B20 производится проверка его наличия
   на шине 1-Wire

01.03.2017
----------
4. Переписана реализация многопоточного измерения (преобразования) температуры
   с использованием класса Thread модуля threading. Старый вариант (с использованием
   модуля thread на Raspberry Pi 3 model B работал нестабильно.

"""
